@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -170,6 +170,32 @@ tracker_data_add_commit_statement_callback (TrackerCommitCallback    callback,
 	g_ptr_array_add (commit_callbacks, delegate);
 }
 
+void
+tracker_data_remove_commit_statement_callback (TrackerCommitCallback callback,
+                                               gpointer              user_data)
+{
+	TrackerCommitDelegate *delegate;
+	guint i;
+	gboolean found = FALSE;
+
+	if (!commit_callbacks) {
+		return;
+	}
+
+	for (i = 0; i < commit_callbacks->len; i++) {
+		delegate = g_ptr_array_index (commit_callbacks, i);
+		if (delegate->callback == callback && delegate->user_data == user_data) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (found) {
+		g_free (delegate);
+		g_ptr_array_remove_index (commit_callbacks, i);
+	}
+}
+
 void
 tracker_data_add_rollback_statement_callback (TrackerCommitCallback    callback,
                                               gpointer                 user_data)
@@ -186,6 +212,33 @@ tracker_data_add_rollback_statement_callback (TrackerCommitCallback    callback,
 	g_ptr_array_add (rollback_callbacks, delegate);
 }
 
+
+void
+tracker_data_remove_rollback_statement_callback (TrackerCommitCallback callback,
+                                                 gpointer              user_data)
+{
+	TrackerCommitDelegate *delegate;
+	guint i;
+	gboolean found = FALSE;
+
+	if (!rollback_callbacks) {
+		return;
+	}
+
+	for (i = 0; i < rollback_callbacks->len; i++) {
+		delegate = g_ptr_array_index (rollback_callbacks, i);
+		if (delegate->callback == callback && delegate->user_data == user_data) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (found) {
+		g_free (delegate);
+		g_ptr_array_remove_index (rollback_callbacks, i);
+	}
+}
+
 void
 tracker_data_add_insert_statement_callback (TrackerStatementCallback callback,
                                             gpointer                 user_data)
@@ -202,6 +255,32 @@ tracker_data_add_insert_statement_callback (TrackerStatementCallback callback,
 	g_ptr_array_add (insert_callbacks, delegate);
 }
 
+void
+tracker_data_remove_insert_statement_callback (TrackerStatementCallback callback,
+                                               gpointer                 user_data)
+{
+	TrackerStatementDelegate *delegate;
+	guint i;
+	gboolean found = FALSE;
+
+	if (!insert_callbacks) {
+		return;
+	}
+
+	for (i = 0; i < insert_callbacks->len; i++) {
+		delegate = g_ptr_array_index (insert_callbacks, i);
+		if (delegate->callback == callback && delegate->user_data == user_data) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (found) {
+		g_free (delegate);
+		g_ptr_array_remove_index (insert_callbacks, i);
+	}
+}
+
 void
 tracker_data_add_delete_statement_callback (TrackerStatementCallback callback,
                                             gpointer                 user_data)
@@ -218,6 +297,32 @@ tracker_data_add_delete_statement_callback (TrackerStatementCallback callback,
 	g_ptr_array_add (delete_callbacks, delegate);
 }
 
+void
+tracker_data_remove_delete_statement_callback (TrackerStatementCallback callback,
+                                               gpointer                 user_data)
+{
+	TrackerStatementDelegate *delegate;
+	guint i;
+	gboolean found = FALSE;
+
+	if (!delete_callbacks) {
+		return;
+	}
+
+	for (i = 0; i < delete_callbacks->len; i++) {
+		delegate = g_ptr_array_index (delete_callbacks, i);
+		if (delegate->callback == callback && delegate->user_data == user_data) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (found) {
+		g_free (delegate);
+		g_ptr_array_remove_index (delete_callbacks, i);
+	}
+}
+
 GQuark tracker_data_error_quark (void) {
 	return g_quark_from_static_string ("tracker_data_error-quark");
 }
@@ -2363,21 +2468,22 @@ ontology_statement_insert (GList       *ontology_queue,
 }
 
 void
-tracker_data_replay_journal (GHashTable *classes,
-                             GHashTable *properties,
-                             GHashTable *id_uri_map)
+tracker_data_replay_journal (GHashTable          *classes,
+                             GHashTable          *properties,
+                             GHashTable          *id_uri_map,
+                             TrackerBusyCallback  busy_callback,
+                             gpointer             busy_user_data,
+                             const gchar         *busy_status)
 {
 	GError *journal_error = NULL;
-	static TrackerProperty *rdf_type = NULL;
+	TrackerProperty *rdf_type = NULL;
 	gint last_operation_type = 0;
 	gboolean in_ontology = FALSE;
 	GList *ontology_queue = NULL;
 
 	tracker_data_begin_db_transaction_for_replay (0);
 
-	if (!rdf_type) {
-		rdf_type = tracker_ontologies_get_property_by_uri (RDF_PREFIX "type");
-	}
+	rdf_type = tracker_ontologies_get_property_by_uri (RDF_PREFIX "type");
 
 	tracker_db_journal_reader_init (NULL);
 
@@ -2647,6 +2753,12 @@ tracker_data_replay_journal (GHashTable *classes,
 				g_warning ("Journal replay error: 'property with ID %d doesn't exist'", predicate_id);
 			}
 		}
+
+		if (busy_callback) {
+			busy_callback (busy_status,
+			               tracker_db_journal_reader_get_progress (),
+			               busy_user_data);
+		}
 	}
 
 	if (journal_error) {
@@ -2665,5 +2777,4 @@ tracker_data_replay_journal (GHashTable *classes,
 	}
 
 	tracker_data_commit_db_transaction ();
-
 }
